package cn.ninegame.genericframework.module;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import cn.ninegame.genericframework.basic.ControllerCenter;
import cn.ninegame.genericframework.basic.ControllerData;
import cn.ninegame.genericframework.basic.GundamFragment;
import cn.ninegame.genericframework.tools.LOG;
import cn.ninegame.genericframework.ui.FragmentCenter;

/**
 * Created by lingcong on 2015/6/2
 * 管理和维护Framework中的所有模块.
 */
public class ModuleCenter {

    private ControllerCenter mControllerCenter = null;
    private FragmentCenter mFragmentCenter = null;

    private HashMap<String,IModuleEntry> mModuleMap = new HashMap<String,IModuleEntry>(2);

    private IModuleEntry mDefaultModuleEntry = null;


    private Handler mModuleHandler = null;
    private ExecutorService mThreadPoolExecutor = Executors.newSingleThreadExecutor();
    private ILoadModuleListener mLoadModuleListener;

    //消息定义
    public static final int MSG_LOAD_MODULE = 1001;
    // 执行缓存操作
    public static final int MSG_TRIGGER_CACHE = 1002;


    private void initHandler(){
        //使用主线程去读，如果这里开线程，可能存在异步线程去读同一变量的情况（线程队列有2个相同的task，前一个load完没写过去，后面的线程就读）
        mModuleHandler = new Handler(Looper.getMainLooper(), new Handler.Callback() {
            @Override
            public boolean handleMessage(Message msg) {
                switch (msg.what){
                    case MSG_LOAD_MODULE:
                        ModuleInfo moduleInfo = (ModuleInfo) msg.obj;
                        final AbsModuleEntry moduleEntry = (AbsModuleEntry)getModuleEntry(moduleInfo.getID());
                        LOG.i("ModuleLoader", "submit load task:" + moduleInfo.getID());

                        mThreadPoolExecutor.submit(new Runnable() {
                            @Override
                            public void run() {
                                boolean isSuccess = moduleEntry.loadModule();
                                if(isSuccess){
                                    Message message = Message.obtain();
                                    message.what = ModuleCenter.MSG_TRIGGER_CACHE;
                                    message.obj = moduleEntry;
                                    mModuleHandler.sendMessage(message);
                                }
                            }
                        });
                        break;
                    case MSG_TRIGGER_CACHE:
                        AbsModuleEntry entry = (AbsModuleEntry) msg.obj;
                        entry.triggerCache();
                        break;
                }
                return false;
            }
        });
    }

    public void setLoadModuleListener(ILoadModuleListener loadModuleListener){
        this.mLoadModuleListener = loadModuleListener;
    }

    public void setControllerCenter (ControllerCenter center) {
        mControllerCenter = center;
    }

    public void setFragmentCenter(FragmentCenter fragmentCenter) {
        mFragmentCenter = fragmentCenter;
    }

    public boolean init(IModuleManifest[] moduleManifests) {
        if (moduleManifests == null || moduleManifests.length == 0) {
            return false;
        }
        //初始化Hander
        initHandler();

        for (int i = 0;i < moduleManifests.length; i++) {
            IModuleManifest moduleManifest = moduleManifests[i];
            AbsModuleEntry moduleEntry = null;
            switch (moduleManifest.getModuleType()) {
                case ModuleType.LOCAL:
                    moduleEntry = new ModuleEntryLocal();
                    if(mDefaultModuleEntry == null) {
                        mDefaultModuleEntry = moduleEntry;
                    }
                    break;

                case ModuleType.DYNAMIC:
                    moduleEntry = new ModuleEntryDynamic();
                    moduleEntry.setModuleHandler(mModuleHandler);
                    moduleEntry.addLoadModuleListener(mLoadModuleListener);
                    break;
            }

            if(moduleEntry == null) {
                continue;
            }

            moduleEntry.setBaseContext(mControllerCenter.getEnviroment().getApplicationContext());
            moduleEntry.setApplicationID(moduleManifest.getModuleApplication());

            moduleEntry.setModuleInfo(moduleManifest.getModuleInfo());
            mModuleMap.put(moduleManifest.getModuleInfo().getID(),moduleEntry);

            ControllerData[] controllers = moduleManifest.getControllerDatas();
            mControllerCenter.registerModuleControllers(moduleManifest.getModuleInfo().getID(),controllers);

            GundamFragment[] fragmentIDs = moduleManifest.getFragmentIDs();
            mFragmentCenter.registerModuleFragment(moduleManifest.getModuleInfo().getID(),fragmentIDs);

            initHandler();
        }

        return true;
    }

    public IModuleEntry getModuleEntry(String moduleID) {
        if (TextUtils.isEmpty(moduleID)) {
            return null;
        }

        return  mModuleMap.get(moduleID);
    }

    public IModuleEntry getDefaultModuleEntry() {
        if(mDefaultModuleEntry == null) {
            mDefaultModuleEntry = new ModuleEntryLocal();
        }

        return  mDefaultModuleEntry;
    }

    public void destroy() {
        Collection<IModuleEntry> moduleEntries = mModuleMap.values();

        Iterator<IModuleEntry> iterator = moduleEntries.iterator();
        while (iterator.hasNext()) {
            IModuleEntry moduleEntry = iterator.next();
            if (moduleEntry != null) {
                moduleEntry.destroy();
            }
        }

        mModuleMap.clear();
    }

    /**
     * 是否有模块正在加载
     * @return
     */
    public boolean hasModuleLoading() {
        Iterator<IModuleEntry> it = mModuleMap.values().iterator();
        boolean result = false;
        while (it.hasNext()) {
            if(it.next().getLoadState() == AbsModuleEntry.LOAD_STATE_LOADING) {
                result = true;
                break;
            }
        }

        return result;
    }
}
