package cn.ninegame.genericframework.ui;

import android.annotation.TargetApi;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;
import android.webkit.WebSettings;
import android.webkit.WebView;

import java.util.HashMap;
import java.util.Map;

import cn.ninegame.genericframework.basic.FrameworkFacade;
import cn.ninegame.genericframework.basic.IResultListener;
import cn.ninegame.genericframework.hybrid.INativeAppInterceptor;
import cn.ninegame.genericframework.hybrid.NativeApp;
import cn.ninegame.genericframework.hybrid.NativeAppEventController;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description:  带NativeApp功能的WebView</p>
 * <br/>
 * <p>JS调用Java方法： NativeApp.prompt('[{"message":"m1","args":{},"callbackId":111},{"message":"m2","args":{}}]');</p>
 * <br/>
 * <p>返回JSON格式的字符串</p>
 * <p>异步方法会执行如下回调 javascript: NativeApp.onCallback(callbackid,{"data":"I am data"}</p>
 * <p/>
 * <p>相关协议：http://gitlab.alibaba-inc.com/aligame/noah/issues/2#note_3354232</p>
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/10/25
 */
public class NativeWebView extends WebView {

    public static final String JAVA_INTERFACE_NAME = "NativeApp";

    private IResultListener mEventResultListener;
    /**
     * 访问Url的Http Header
     */
    Map<String, String> mAdditionalHttpHeaders = new HashMap<String, String>();
    /**
     * 是否拦截返回键
     */
    private boolean mIsInterceptBackKey;

    public NativeWebView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public NativeWebView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    @TargetApi(21)
    public NativeWebView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init();
    }

    public NativeWebView(Context context, AttributeSet attrs, int defStyleAttr, boolean privateBrowsing) {
        super(context, attrs, defStyleAttr, privateBrowsing);
        init();
    }

    public NativeWebView(Context context) {
        super(context);
        init();
    }

    private void init() {
        setupWebViewAttributes(this);
        // 注入接口
        this.addJavascriptInterface(new JSInterface(), JAVA_INTERFACE_NAME);

    }

    @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
    public static void setupWebViewAttributes(WebView webView) {
        WebSettings webSettings = webView.getSettings();

        if (!webSettings.getJavaScriptEnabled()) {
            try {
                webSettings.setJavaScriptEnabled(true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        webSettings.setLoadWithOverviewMode(true);
        webSettings.setUseWideViewPort(false);
        webSettings.setBuiltInZoomControls(false);
        webSettings.setSupportZoom(false);
        webSettings.setRenderPriority(WebSettings.RenderPriority.HIGH);
        webSettings.setCacheMode(WebSettings.LOAD_DEFAULT);
        webSettings.setAppCachePath(webView.getContext().getFilesDir() + "/webcache");
        webSettings.setAppCacheEnabled(true);
        webSettings.setLoadsImagesAutomatically(true);
        webSettings.setLightTouchEnabled(false);
        webSettings.setDomStorageEnabled(true); // supports local storage
        webSettings.setDatabaseEnabled(true);   // supports local storage
        webSettings.setDatabasePath(webView.getContext().getFilesDir() + "/localstorage");

        if (Build.VERSION.SDK_INT >= 16) {
            webSettings.setAllowUniversalAccessFromFileURLs(true);
        }

        // we are using ApplicationContext when creaing BrowserTab, without disabling the "Save Password" dialog
        // there will be an exception that would cause crash: "Unable to add window -- token null is not for an application"
        webSettings.setSavePassword(false);
        webView.setHorizontalScrollBarEnabled(false);
//        mWebView.setVerticalScrollBarEnabled(false);
        webView.setScrollbarFadingEnabled(true);
        webView.setScrollBarStyle(View.SCROLLBARS_OUTSIDE_OVERLAY);

        // 启动硬件加速
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            // chromium, enable hardware acceleration
            webView.setLayerType(View.LAYER_TYPE_HARDWARE, null);
        } else {
            // older android version, disable hardware acceleration
            webView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
        }

        // http 和 https 混合使用，导致图片显示不出来
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            webSettings.setMixedContentMode(WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);
        }
    }

    /**
     * 注入WebView的接口
     */
    class JSInterface {
        @android.webkit.JavascriptInterface
        public String prompt(String cmd) {
            Log.d("SimpleWebView", "prompt = " + cmd);
            return NativeApp.prompt(NativeWebView.this, cmd);
        }
    }

    public IResultListener getEventResultListener() {
        if (mEventResultListener == null) {
            mEventResultListener = new IResultListener() {
                @Override
                public void onResult(Bundle result) {
                    NativeApp.callJSOnEvent(NativeWebView.this, result);
                }
            };
        }

        return mEventResultListener;
    }

    @Override
    public void loadUrl(String url) {
        if (mAdditionalHttpHeaders.size() == 0) {
            resetEventIfNeed(url);
            super.loadUrl(url);
        } else {
            this.loadUrl(url, mAdditionalHttpHeaders);
        }
    }

    @Override
    public void loadUrl(String url, Map<String, String> additionalHttpHeaders) {
        resetEventIfNeed(url);
        super.loadUrl(url, additionalHttpHeaders);
    }

    public void addAdditionalHttpHeader(Map<String, String> map){
        mAdditionalHttpHeaders.putAll(map);
    }

    public static void addInterceptor(INativeAppInterceptor interceptor){
        NativeApp.addInterceptor(interceptor);
    }

    public static void removeInterceptor(INativeAppInterceptor interceptor){
        NativeApp.removeInterceptor(interceptor);
    }

    /**
     * 当前 WebView 是否需要拦截返回键
     * @return
     */
    public boolean isInterceptBackKey() {
        return mIsInterceptBackKey;
    }

    public void setInterceptBackKey(boolean intercept){
        mIsInterceptBackKey = intercept;
    }

    private void resetEventIfNeed(String url){
        if (!NativeApp.isJSString(url)){
            // 加载非Js脚本, 清除注册的事件
            resetEvent();
        }
    }

    //清除注册的事件
    private void resetEvent(){
        mIsInterceptBackKey = false;

        FrameworkFacade.getInstance().getEnvironment().sendMessage(NativeAppEventController.MSG_CLEAN_EVENT);
    }

}
